//
//  FLFetchedResultsController.h
//  FLKit
//
//  Created by Frankie Laguna on 1/20/13.
//  Copyright (c) 2013 FIPLAB Ltd. All rights reserved.
//

#import <Foundation/Foundation.h>

@protocol FLFetchedResultsControllerDelegate;

@interface FLFetchedResultsController : NSObject

/* Initializers */
- (id)initWithFetchRequest:(NSFetchRequest *)fetchRequest managedObjectContext: (NSManagedObjectContext *)context sectionNameKeyPath:(NSString *)sectionNameKeyPath cacheName:(NSString *)name;

- (BOOL)performFetch:(NSError *__autoreleasing *)error;

/* Configuration */
@property (nonatomic, readonly) NSFetchRequest *fetchRequest;
@property (nonatomic, readonly) NSManagedObjectContext *managedObjectContext;
@property (nonatomic, readonly) NSString *sectionNameKeyPath;
@property (nonatomic, readonly) NSString *cacheName;
@property (nonatomic, assign) id<FLFetchedResultsControllerDelegate> delegate;

+ (void)deleteCacheWithName:(NSString *)name;

/* Accessing Object Results */
@property (nonatomic, readonly) NSArray *fetchedObjects;

- (id)objectAtIndexPath:(NSIndexPath *)indexPath;

-(NSIndexPath *)indexPathForObject:(id)object;

@end

/* Change Type Enum */
enum {
	FLFetchedResultsChangeInsert = 1,
	FLFetchedResultsChangeDelete = 2,
	FLFetchedResultsChangeMove = 3,
	FLFetchedResultsChangeUpdate = 4
};
typedef NSUInteger FLFetchedResultsChangeType;

/* Protocol */
@protocol FLFetchedResultsControllerDelegate <NSObject>

@optional
- (void)controller:(FLFetchedResultsController *)controller didChangeObject:(id)anObject atIndexPath:(NSIndexPath *)indexPath forChangeType:(FLFetchedResultsChangeType)type newIndexPath:(NSIndexPath *)newIndexPath;
@end